<?php

namespace App\Helpers;

use PHPMailer\PHPMailer\PHPMailer;

class EmailClient
{

    /**
     * Sends an email using PHPMailer.
     *
     * @param string $to The recipient's email address.
     * @param string $name The recipient's name.
     * @param string $subject The subject of the email.
     * @param string $body The body of the email.
     * @throws Exception If an error occurs while sending the email.
     * @return bool|string Returns true if the email was sent successfully, or the error message if there was an error.
     */
    public static function send($to, $name, $subject, $body){

    $mail = new PHPMailer(true);

    try {
        $mail->isSMTP();
        $mail->Host = env('MAIL_HOST');
        $mail->SMTPAuth = true;
        $mail->Username = env('MAIL_USERNAME');
        $mail->Password = env('MAIL_PASSWORD');
        // $mail->SMTPSecure = 'smtp'; // Enable TLS or SSL encryption
        $mail->Port = env('MAIL_PORT'); // Check the port for your SMTP server

        // Sender and recipient details
        $mail->setFrom(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME'));
        $mail->addAddress($to, $name);

        // Email content
        $mail->isHTML(true);
        $mail->Subject = $subject;
        $mail->Body = $body;

        // Attachments (if needed)
        // $mail->addAttachment('path-to-attachment/file.pdf');

        // Send the email
        $mail->send();
        return true;
    } catch (Exception $e) {
        return $mail->ErrorInfo;
    }
    }
}
