<?php

namespace App\Helpers;

use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Client;

class SMSClient
{

    /**
     * Sends an SMS message to the specified phone number.
     *
     * @param string $phone The phone number to send the SMS to.
     * @param string $text The text message to send.
     * @throws Exception If there is an error sending the SMS.
     * @return mixed The response from the SMS service.
     */
    public static function send($phone, $text){
        $client = new Client();
        $query =  http_build_query(
            [
                "username" => env('SMS_USERNAME'),
                "password" => env('SMS_API_PASSWORD'),
                'senderid' => env('SMS_SNEDER_ID'),
                "to"       => $phone,
                "text"     => $text."

----------------------
tasree.ae",
                "datetime" => date('Y-m-d H:i:s', strtotime('now')),
            ]
        );
        $request = new Request('GET', env('SMS_URL').'/api_json.php?'.$query);
        $res = $client->sendAsync($request)->wait();
        return json_decode($res->getBody());
    }

    /**
     * Checks the delivery status of a message.
     *
     * @param string $message_id The ID of the message to check.
     * @throws Exception if there's an error in the API request.
     * @return stdClass The JSON response from the API containing the delivery status.
     */
    public static function checkDelivery($message_id){
        $client = new Client();
        $query =  http_build_query(
            [
                "username" => env('SMS_USERNAME'),
                "password" => env('SMS_API_PASSWORD'),
                'msgid' => $message_id,
            ]);
        $request = new Request('GET', env('SMS_URL').'/api_json_dlr.php?'.$query);
        $res = $client->sendAsync($request)->wait();
        return json_decode($res->getBody());
    }

}
