<?php

namespace App\Http\Controllers;

use App\Models\Place;
use App\Models\User;
use App\Models\Ticket;
use App\Models\Ads;
use App\Models\Assignment;
use App\Models\Promotion;
use App\Models\CarDamage;
use Illuminate\Http\Request;
use App\Rules\ArrayValuesInList;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Hash;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Broadcasting\BroadcastException;
use Twilio\Rest\Client;
use Illuminate\Database\Eloquent\Builder;
use Exception;
use App\Helpers\SMSClient;
use App\Helpers\PaymentClient;
use App\Helpers\EmailClient;
use App\Helpers\PusherClient;
use App\Enums\CarStatus;
use AshAllenDesign\ShortURL\Facades\ShortURL;
use Illuminate\Support\Carbon;

class UserController extends Controller
{

    /**
     * Get the user places.
     *
     * @throws Exception if an error occurs.
     * @return Response with the user places.
     */
    public function getUserPlaces(){
        try{
        return response()->standard("",[
            "default_place" => auth('sanctum')->user()->default_place,
            "places" => auth()->user()->places->map(function($place){
                return collect($place->toArray())
                    ->only(['id', 'name', 'logo', 'url'])
                    ->all();
            }),
        ]);
        }catch(Exception $e){
            return response()->error(1002, "Some Error accured", $e->getMessage());
        }
    }

    /**
     * Set the default place for the user.
     *
     * @param Request $request The request object.
     * @throws Some_Exception_Class description of exception
     * @return Some_Return_Value
     */
    public function setDefaultPlace(Request $request){
        $validator = Validator::make($request->all(), [
            'place_uuid' => ['required', 'exists:places,id']
        ]);
        if($validator->fails())
            return response()->validation($validator->errors());

        $data = $validator->validated();
        if(!in_array($data['place_uuid'], auth()->user()->places->pluck('id')->toArray())){
            return response()->error(403, "not authorized1", []);
        }
        $user = auth('sanctum')->user();
        $user->default_place = $data['place_uuid'];
        $user->save();
        $user->tokens()->delete();
        $token = $user->createToken($user->username)->plainTextToken;
        return response()->standard("", ["token" => $token]);
    }

}


