<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Concerns\HasUuids;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Notifications\Notifiable;

class Advertisement extends Model
{

    use SoftDeletes, HasUuids, Notifiable;
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'url',
        'image',
        'expiration'
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'expiration' => 'date',
    ];


    /**
     * Get the places that belong to the ads.
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function places(): BelongsToMany
    {
        return $this->belongsToMany(
            Place::class,                       // The related model
            'place_ads',                           // The pivot table
            'ad_uuid',                      // The foreign key of this model
            'place_uuid'                        // The foreign key of the related model
        )->using(PlaceAds::class)                  // The intermediate model
        ->withPivot('is_active')              // The pivot columns to retrieve
        ->withTimestamps();                    // The timestamp columns
    }

}
