<?php

namespace App\Nova\Actions;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Collection;
use Laravel\Nova\Actions\Action;
use Laravel\Nova\Fields\ActionFields;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Fields\HasMany;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Number;
use Laravel\Nova\Fields\Boolean;
use Laravel\Nova\Fields\Date;
use Lednerb\ActionButtonSelector\ShowAsButton;
use App\Models\Promotion;

class Promote extends Action
{
    use InteractsWithQueue, Queueable;
    use ShowAsButton;
    public $name = "Validation";

    /**
     * Perform the action on the given models.
     *
     * @param  \Laravel\Nova\Fields\ActionFields  $fields
     * @param  \Illuminate\Support\Collection  $models
     * @return mixed
     */
    public function handle(ActionFields $fields, Collection $models)
    {
        foreach($models as $model){
            $promote = new Promotion([
                "text" => $fields["text"],
                "value" => $fields["value"],
                "exp" => $fields["exp"],
                "status" => $fields["status"],
                "place_uuid" => $model->id,
            ]);
            $promote->save();
        }
    }

    /**
     * Get the fields available on the action.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        return [
            Text::make('Text', 'text')->maxlength(10),
            Number::make('Value', 'value')->min(1),
            Number::make('Usage Count', 'usage_count')->min(1)->step(1),
            Boolean::make('Status','status'),
            Date::make('Exp.', 'exp')->filterable(),
        ];
    }
}