<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Number;
use SimpleSquid\Nova\Fields\Enum\Enum;
use Laravel\Nova\Fields\Boolean;
use Laravel\Nova\Fields\HasOne;
use Laravel\Nova\Fields\HasMany;
use Laravel\Nova\Fields\BelongsTo;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Panel;
use DigitalCreative\ColumnToggler\ColumnTogglerTrait;
use App\Enums\CarStatus;
use App\Nova\Actions\AssignTo;
use App\Nova\Actions\RecieveCash;
use Illuminate\Support\Facades\DB;
use Laravel\Nova\Fields\Date;
use Laravel\Nova\Fields\DateTime;
use Laravel\Nova\Fields\Email;
use Pavloniym\ActionButtons\ActionButton;
use Pavloniym\ActionButtons\ActionButtons;
use App\Http\Controllers\CustomerController;
use Laravel\Nova\Fields\Select;

use function Laravel\Prompts\text;

// use Coroowicaksono\ChartJsIntegration\LineChart;

class Assignment extends Resource
{
    use ColumnTogglerTrait;


    /**
     * Returns the label for the function.
     *
     * @return string The label for the function.
     */
    public static function label() {
        return 'Cars';
    }
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\Assignment>
     */
    public static $model = \App\Models\Assignment::class;

    public static $trafficCop = false;

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'ticket.label';

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'ticket.label','car_plate','car_make',
    ];

    /**
     * Get the fields displayed by the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        return [
            ID::make('uuid')->hidden()->sortable(),
            Text::make('Place')->hideWhenUpdating(),
            Belongsto::make('Ticket', 'ticket', Ticket::class)->readOnly()->searchable(),
            Text::make('Sms Id', 'sms_id'),
            Text::make('Country Code', 'country_code')->hideFromIndex(),
            Number::make('Phone', 'phone')->hideFromIndex(),
            Email::make('Email', 'email')->hideFromIndex(),
            Enum::make('Status','status')->hideWhenUpdating()->displayUsing(function($value){
                if($value == 'parking')
                    return 'Check In';
                else
                    return $value;
            }),
            Enum::make('Status','assignments.status')->hideFromIndex()
                    ->hideWhenUpdating()
                    ->hideFromDetail()
                    ->attach(CarStatus::class)
                    ->filterable(),
            Text::make('Car Plate', 'car_plate')->hideWhenUpdating()->readOnly(),
            Select::make('Car Color', 'car_color')
            ->options(array_keys(config('color')))
            ->displayUsingLabels(),
            Select::make('Car Make', 'car_make')
            ->options(array_values(config('car-brands')))
            ->displayUsingLabels(),
            DateTime::make('Entrance','created_at')->hideWhenUpdating()->displayUsing(fn ($value) => $value ? $value->format('d M Y,') . ' ' . $value->format('h:i A') : ' '),
            DateTime::make('Last Update','updated_at')->hideWhenUpdating()->displayUsing(fn ($value) => $value ? $value->format('d M Y,') . ' ' . $value->format('h:i A') : ' '),
            Boolean::make("Carwash"),
            Boolean::make("Fuel"),
            Boolean::make("Paid")->hideWhenUpdating(),
            Number::make('Extra Charge', 'extra_service')->hideFromIndex(),
            Number::make('ُTip', 'tip')->hideFromIndex(),
            Number::make('Discount', 'discount')->hideFromIndex(),
            // Text::make('Bill', 'price'),
            (new Panel('Guest', $this->guest())),
            HasMany::make('Damages', 'carDamages', 'App\Nova\CarDamage'),
            Boolean::make('Damages', 'hasDamage')->hideWhenUpdating(),
            (new Panel('Valet', $this->userFields())),
            ActionButton::make('Assign To')->icon('<svg xmlns="http://www.w3.org/2000/svg" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor" class="w-6 h-6">
            <path stroke-linecap="round" stroke-linejoin="round" d="M18 7.5v3m0 0v3m0-3h3m-3 0h-3m-2.25-4.125a3.375 3.375 0 1 1-6.75 0 3.375 3.375 0 0 1 6.75 0ZM3 19.235v-.11a6.375 6.375 0 0 1 12.75 0v.109A12.318 12.318 0 0 1 9.374 21c-2.331 0-4.512-.645-6.374-1.766Z" />
          </svg>
          ')->styles([
            'background-color' => 'rgba(0, 0, 0, 0)',
            'shadow' => 'none',
          ])
          ->action(new AssignTo($this->resource), $this->resource->id),
          ActionButton::make('Recieve Cash')->icon('<svg xmlns="http://www.w3.org/2000/svg" class="h-6 w-6" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke="currentColor" stroke-width="1.5">
          <path stroke-linecap="round" stroke-linejoin="round" d="M17 9V7a2 2 0 00-2-2H5a2 2 0 00-2 2v6a2 2 0 002 2h2m2 4h10a2 2 0 002-2v-6a2 2 0 00-2-2H9a2 2 0 00-2 2v6a2 2 0 002 2zm7-5a2 2 0 11-4 0 2 2 0 014 0z" />
        </svg>
          ')->styles([
            'background-color' => 'rgba(0, 0, 0, 0)',
            'shadow' => 'none',
          ])
          ->action((new RecieveCash($this->resource))->confirmText('Customer`s payment has been received. Are you sure?')
          ->confirmButtonText('Paid')
          ->cancelButtonText("Not Paid"), $this->resource->id)
        ];
    }


    protected function guest()
    {
        return [
            Boolean::make("Guest", "is_guest")->filterable(),
            Text::make("Room", "room")->hideFromIndex(),
            Date::make("Check Out", "checkout_date")->hideFromIndex(),
        ];
    }
    /**
     * Get the valet fields for the resource.
     *
     * @return array
     */
    protected function userFields()
    {
        return [
            BelongsTo::make('Transferee', 'user', User::class)->showOnDetail()->withoutTrashed()->readOnly()->hideWhenUpdating(),
            BelongsTo::make('Assigned To', 'assignedTo', User::class)->showOnDetail()->nullable()->withoutTrashed()->hideWhenUpdating(),
            BelongsTo::make('Cash Reciever', 'cashReciever', User::class)->showOnDetail()->nullable()->withoutTrashed()->hideWhenUpdating(),
            Text::make('Level', 'level')->showOnDetail()->hideFromIndex(),
            Text::make('Slot',  'slot')->showOnDetail()->hideFromIndex(),
            Text::make('Hanger', 'hanger')->showOnDetail()->hideFromIndex(),
        ];
    }
    /**
     * Get the cards available for the request.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function cards(NovaRequest $request)
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function filters(NovaRequest $request)
    {
        return [
            new \App\Nova\Filters\PlaceFilter(),
            new \App\Nova\Filters\FromDateFilter(),
            new \App\Nova\Filters\ToDateFilter(),
        ];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function lenses(NovaRequest $request)
    {
        return [
            (new Lenses\RequestedCars),
            (new Lenses\GuestCars),
        ];
    }

    /**
     * Get the actions available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function actions(NovaRequest $request)
    {
        return [
            (new \App\Nova\Actions\AssignTo($this->resource))->sole(),
            (new \App\Nova\Actions\RecieveCash),
        ];
    }

        /**
     * Build a "relatable" query for the given resource.
     *
     * This query determines which instances of the model may be attached to other resources.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @param  \Illuminate\Database\Eloquent\Builder  $query
     * @param  \Laravel\Nova\Fields\Field  $field
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public static function relatableUsers(NovaRequest $request, $query)
    {
        {
            $users = DB::table('roles')->whereJsonContains('permissions', 'VALET')->pluck('user_uuid')->toArray();
            if(\App\Models\Assignment::find($request->resourceId))
                return $query->join('roles', 'users.id', '=', 'roles.user_uuid')
                    ->whereIn('users.id', $users)->where('roles.place_uuid', '=', \App\Models\Assignment::find($request->resourceId)->ticket->place->id)
                    ->select('users.username');
            return $query;
        }
    }

    public static function indexQuery(NovaRequest $request, $query)
    {
        if($request->user()->superadmin){
            return $query;
        }
            return $query
                ->join('tickets', 'assignments.ticket_uuid', '=', 'tickets.id')
                ->whereIn('tickets.place_uuid', $request->user()->places->pluck('id')->toArray())
                ->select('assignments.*');

    }
}
