<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Trix;
use Laravel\Nova\Fields\Number;
use Laravel\Nova\Fields\Avatar;
use Laravel\Nova\Http\Requests\NovaRequest;
use SimpleSquid\Nova\Fields\Enum\Enum;
use Outl1ne\MultiselectField\Multiselect;
use App\Enums\PlaceType;
use App\Enums\RoleType;
use App\Nova\Actions\ImportTicketCsv;
use App\Nova\Actions\Manytickets;
use App\Nova\Actions\CashRecievedAction;
use App\Nova\Actions\Promote;
use Laravel\Nova\Fields\BelongsToMany;
use Laravel\Nova\Fields\HasMany;
use Laravel\Nova\Panel;
use DigitalCreative\ColumnToggler\ColumnTogglerTrait;
use Laravel\Nova\Fields\Boolean;
use YieldStudio\NovaPhoneField\PhoneNumber;
use Laravel\Nova\Fields\Country;
use Laravel\Nova\Fields\Timezone;
use Pavloniym\ActionButtons\ActionButton;
use Pavloniym\ActionButtons\ActionButtons;

class Place extends Resource
{
    use ColumnTogglerTrait;
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\Place>
     */
    public static $model = \App\Models\Place::class;

    public static $trafficCop = false;
    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'name';
    public static function label() {
        return 'Properties';
    }
    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'name','country_code',
    ];

    /**
     * Get the fields displayed by the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        return [
            ID::make('uuid')->hidden(),
            Avatar::make('Logo')->disk('hotelimages'),
            Text::make('Name')->filterable()->rules('required'),
            Enum::make('Type')->attach(PlaceType::class)->filterable(),

            Trix::make('Description')
                ->showWhenPeeking(),
            Text::make('Customer Message', 'txt_msg')->hideFromIndex(),
            Country::make('Country', 'country_code')->required(),
            Timezone::make('Timezone', 'time_zone')->required()->default('UTC'),
            PhoneNumber::make('Phone'),
            Number::make('Capacity')->hideFromIndex(),
            Number::make('Parking Availability', 'parking_availability')->hideFromIndex()->hideWhenCreating()->hideWhenUpdating(),
            (new Panel('Price List', $this->PriceFields()))->limit(1),
            HasMany::make("Validations","promotions", Promotion::class),
            BelongsToMany::make('Users')->fields(function ($request, $relatedModel) {
                return [
                    MultiSelect::make('Permissions')
                        ->options(RoleType::asArray())
                        ->saveAsJSON(),
                ];
            }),

            BelongsToMany::make('Advertisements')->fields(function ($request, $relatedModel) {
                return [
                    Boolean::make('Status', 'is_active'),
                ];
            }),

            Number::make('Tickets', function(){
                return $this->tickets->count();
            }),

            HasMany::make("Tickets"),

            ActionButtons::make('')->collection([
            ActionButton::make('Validation')->icon('<svg xmlns="http://www.w3.org/2000/svg" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor" class="w-6 h-6">
            <path stroke-linecap="round" stroke-linejoin="round" d="M2.25 18.75a60.07 60.07 0 0 1 15.797 2.101c.727.198 1.453-.342 1.453-1.096V18.75M3.75 4.5v.75A.75.75 0 0 1 3 6h-.75m0 0v-.375c0-.621.504-1.125 1.125-1.125H20.25M2.25 6v9m18-10.5v.75c0 .414.336.75.75.75h.75m-1.5-1.5h.375c.621 0 1.125.504 1.125 1.125v9.75c0 .621-.504 1.125-1.125 1.125h-.375m1.5-1.5H21a.75.75 0 0 0-.75.75v.75m0 0H3.75m0 0h-.375a1.125 1.125 0 0 1-1.125-1.125V15m1.5 1.5v-.75A.75.75 0 0 0 3 15h-.75M15 10.5a3 3 0 1 1-6 0 3 3 0 0 1 6 0Zm3 0h.008v.008H18V10.5Zm-12 0h.008v.008H6V10.5Z" />
          </svg>
          ')->styles([
            'background-color' => 'rgba(0, 0, 0, 0)',
            'shadow' => 'none',
          ])
          ->action(new Promote($this->resource), $this->resource->id),

            ActionButton::make('Import Ticket CSV')->icon('<svg xmlns="http://www.w3.org/2000/svg" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor" class="w-6 h-6">
            <path stroke-linecap="round" stroke-linejoin="round" d="M12 10.5v6m3-3H9m4.06-7.19-2.12-2.12a1.5 1.5 0 0 0-1.061-.44H4.5A2.25 2.25 0 0 0 2.25 6v12a2.25 2.25 0 0 0 2.25 2.25h15A2.25 2.25 0 0 0 21.75 18V9a2.25 2.25 0 0 0-2.25-2.25h-5.379a1.5 1.5 0 0 1-1.06-.44Z" />
          </svg>
            ')->styles([
                'background-color' => 'rgba(0, 0, 0, 0)',
                'shadow' => 'none',
            ])
            ->action(new ImportTicketCsv($this->resource), $this->resource->id),

            ActionButton::make('Ticket Range')->icon('<svg xmlns="http://www.w3.org/2000/svg" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor" class="w-6 h-6">
            <path stroke-linecap="round" stroke-linejoin="round" d="M16.5 6v.75m0 3v.75m0 3v.75m0 3V18m-9-5.25h5.25M7.5 15h3M3.375 5.25c-.621 0-1.125.504-1.125 1.125v3.026a2.999 2.999 0 0 1 0 5.198v3.026c0 .621.504 1.125 1.125 1.125h17.25c.621 0 1.125-.504 1.125-1.125v-3.026a2.999 2.999 0 0 1 0-5.198V6.375c0-.621-.504-1.125-1.125-1.125H3.375Z" />
          </svg>
          ')->styles([
            'background-color' => 'rgba(0, 0, 0, 0)',
            'shadow' => 'none',
          ])
          ->action(new Manytickets($this->resource), $this->resource->id),
          ActionButton::make('Cash Report')->icon('<svg xmlns="http://www.w3.org/2000/svg" fill="none" color="rgb(148, 163,184)" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor" class="w-6 h-6">
          <path stroke-linecap="round" stroke-linejoin="round" d="M3 10h18M3 14h18m-9-4v8m-7 0h14a2 2 0 002-2V8a2 2 0 00-2-2H5a2 2 0 00-2 2v8a2 2 0 002 2z" />
        </svg>')->styles([
          'background-color' => 'rgba(0, 0, 0, 0)',
          'shadow' => 'none',
        ])
        ->action(new CashRecievedAction($this->resource), $this->resource->id),

          ]),

        ];
    }
    /**
     * Get the price fields for the resource.
     *
     * @return array
     */
    protected function priceFields()
    {
        return [
            Number::make('Tax Price', 'tax')->hideFromIndex()->rules('required'),
            Number::make('All Week Price', 'all_week_price')->hideFromIndex()->rules('required')->step(0.01),
            Number::make('Week Days Price (Mon - Thu)', 'week_days_price')->hideFromIndex()->step(0.01),
            Number::make('Friday Price', 'friday_price')->hideFromIndex()->step(0.01),
            Number::make('Weekend Price', 'weekend_price')->hideFromIndex()->step(0.01),
            Number::make('Carwash Price', 'carwash_price')->hideFromIndex()->step(0.01),
            Number::make('Fuel Price', 'fuel_price')->hideFromIndex()->step(0.01),
        ];
    }
    /**
     * Get the cards available for the request.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function cards(NovaRequest $request)
    {
        return [];
    }
    /**
     * Exclude attributes from import.
     *
     * @return array The excluded attributes.
     */
    public static function excludeAttributesFromImport()
    {
        return ['id'];
    }
    /**
     * Get the filters available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function filters(NovaRequest $request)
    {
        return [
            new \App\Nova\Filters\PlaceFilter(),
        ];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function lenses(NovaRequest $request)
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function actions(NovaRequest $request)
    {
        return [
            (new Actions\Promote),
            (new Actions\ImportTicketCsv),
            (new Actions\Manytickets),
            (new Actions\CashRecievedAction),
        ];
    }
    /**
     * Filters the query based on the user's permissions.
     *
     * @param NovaRequest $request The request object.
     * @param mixed $query The query to be filtered.
     * @return mixed The filtered query.
     */
    public static function indexQuery(NovaRequest $request, $query)
    {
        if($request->user()->superadmin){
            return $query;
        }
            return $query
                ->whereIn('places.id', $request->user()->places->pluck('id')->toArray());

    }
}
