<?php

namespace App\Nova\Widgets;

use DigitalCreative\NovaDashboard\Filters;
use DigitalCreative\TableWidget\TableWidget;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Number;
use Laravel\Nova\Fields\Avatar;
use App\Models\Place;
use App\Models\Ticket;
use App\Models\Assignment;
use Laravel\Nova\Http\Requests\NovaRequest;
use App\Nova\Filters\FromDateFilter;
use App\Nova\Filters\ToDateFilter;
use App\Nova\Filters\PlaceFilter;
use Illuminate\Support\Carbon;


class IncomeTableWidget extends TableWidget
{
    /**
     * Configure the function.
     *
     * @param NovaRequest $request The NovaRequest parameter.
     * @throws None
     * @return void
     */
    public function configure(NovaRequest $request): void
    {

        $this->title('Places Income');
        $this->minWidth(4);
        $this->minHeight(2);

    }

    /**
     * Returns an array of fields for the function.
     *
     * @return array
     */
    public function fields(): array
    {
        return [
            Avatar::make('logo'),
            Text::make('Name'),
            Text::make('type'),
            Number::make('Count'),
            Number::make('Income'),

        ];
    }

    /**
     * Retrieves the value from the given Filters object.
     *
     * @param Filters $filters The Filters object to retrieve the value from.
     * @return \Illuminate\Support\Collection The collection of values retrieved from the Place model.
     */
    public function value(Filters $filters)
    {
        $fromValue = $filters->getFilterValue(FromDateFilter::class);
        $toValue = $filters->getFilterValue(ToDateFilter::class);
        // if($fromValue == null)
        //     $fromValue = date("Y-m-d H:i:s", strtotime("-30 days"));
        return Place::get()->map(function ($place) use ($fromValue, $toValue){
            return [
                'name' => $place->name,
                'count' => Assignment::whereIn('ticket_uuid', Ticket::where('place_uuid', $place->id)->pluck('id'))
                    ->where('created_at', ">=", Carbon::parse($fromValue))
                    ->where('created_at', "<=", Carbon::parse($toValue))
                    ->pluck('charge')->count(),
                'income' => strval(Assignment::whereIn('ticket_uuid', Ticket::where('place_uuid', $place->id)->pluck('id'))
                    ->where('created_at', ">=", Carbon::parse($fromValue))
                    ->where('created_at', "<=", Carbon::parse($toValue))
                    ->pluck('charge')->sum()) . " AED",
                'logo' => "hotels/".$place->logo,
                'type' => function() use ($place){
                    if($place->type == "hotel")
                        return 'Hotel';
                        // 'icons/hotel.png'
                    if($place->type == "mosque")
                        return 'Mosque';
                        // 'icons/mosque.png'
                    if($place->type == "restaurant")
                        return 'Restaurant';
                    // 'icons/restaurant.png'
                    if($place->type == "club")
                        return 'Club';
                        // 'icons/club.png'
                    if($place->type == "hotelresidence")
                        return 'Hotel Residence';
                        // 'icons/hotelresidence.png'
                    if($place->type == "residence")
                        return 'Residence';
                    if($place->type == "mall")
                        return 'Mall';
                    else
                        return 'Other';
                        // 'HP32keQ3iV75h5cvuJstckyslxfiGtJ2eSE8tMOd.jpg'
                    }
            ];

        });
        // return $filters->getFilterValue($result);
    }

}
