<?php

namespace App\Policies;

use App\Models\Assignment;
use App\Models\User;
use Illuminate\Auth\Access\Response;

class AssignmentPolicy
{
    /**
     * Determine whether the user can view any models.
     */
    public function viewAny(User $user): bool
    {
        return true;//$user->is_superadmin | $user->is_manager;
    }

    /**
     * Determine whether the user can view the model.
     */
    public function view(User $user, Assignment $assignment): bool
    {
        if($user->superadmin)
            return true;

        $place_uuid = $assignment->ticket->place->id;
        if($user->places->find($place_uuid) == null)
            return false;
        
        return (
                    $user->places->find($place_uuid)->pivot->permissions->contains("MANAGER")   
                    ||
                    $user->places->find($place_uuid)->pivot->permissions->contains("VALET_DESK")
                    ||
                    $user->places->find($place_uuid)->pivot->permissions->contains("RECEPTION")
                    // (
                    //     $user->places->find($place_uuid)->pivot->permissions->contains("VALET") 
                    //     && 
                    //     $assignment->assigned_to == null)
                );
    }

    /**
     * Determine whether the user can create models.
     */
    public function create(User $user): bool
    {
        return false;
    }

    /**
     * Determine whether the user can update the model.
     */
    public function update(User $user, Assignment $assignment): bool
    {
        if($user->superadmin)
            return true;

        $place_uuid = $assignment->ticket->place->id;
        if($user->places->find($place_uuid) == null)
            return false;
        
        return (
                    $user->places->find($place_uuid)->pivot->permissions->contains("MANAGER")   
                    ||
                    $user->places->find($place_uuid)->pivot->permissions->contains("VALET_DESK")
                    ||
                    $user->places->find($place_uuid)->pivot->permissions->contains("RECEPTION")
                    // (
                    //     $user->places->find($place_uuid)->pivot->permissions->contains("VALET") 
                    //     && 
                    //     $assignment->assigned_to == null)
                );
    }

    /**
     * Determine whether the user can delete the model.
     */
    public function delete(User $user, Assignment $assignment): bool
    {
        if($user->superadmin)
            return true;

        $place_uuid = $assignment->ticket->place->id;
        if($user->places->find($place_uuid) == null)
            return false;
        
        return (
                    $user->places->find($place_uuid)->pivot->permissions->contains("MANAGER")   
                    ||
                    $user->places->find($place_uuid)->pivot->permissions->contains("VALET_DESK")
                    // (
                    //     $user->places->find($place_uuid)->pivot->permissions->contains("VALET") 
                    //     && 
                    //     $assignment->assigned_to == null)
                );
    }

    /**
     * Determine whether the user can restore the model.
     */
    public function restore(User $user, Assignment $assignment): bool
    {
        return false;
    }

    /**
     * Determine whether the user can permanently delete the model.
     */
    public function forceDelete(User $user, Assignment $assignment): bool
    {
        return false;
    }
}
