<?php

namespace App\Providers;

use App\Nova\Advertisement;
use App\Nova\User;
use App\Nova\Place;
use App\Nova\Ticket;
use App\Nova\Assignment;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Gate;
use Laravel\Nova\Nova;
use Laravel\Nova\NovaApplicationServiceProvider;
use Laravel\Nova\Dashboards\Main;
use Laravel\Nova\Dashboards\Income;
use Laravel\Nova\Dashboards\Parking;
use Laravel\Nova\Menu\Menu;
use Laravel\Nova\Menu\MenuItem;
use Laravel\Nova\Menu\MenuSection;
use Laravel\Nova\Menu\MenuGroup;
use Illuminate\Http\Request;
use App\Providers\NovaRequest;
use SimonHamp\LaravelNovaCsvImport\LaravelNovaCsvImport;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Blade;

class NovaServiceProvider extends NovaApplicationServiceProvider
{

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        parent::boot();
        Nova::withoutNotificationCenter();
        Nova::createUserUsing(function(Command $command) {
            return [
                $command->ask('Username'),
                $command->ask('Email Address'),
                $command->secret('Password'),
            ];
        }, function($name, $email, $password) {
            (new \App\Models\User())->forceFill([
                'username' => $name,
                'email' => $email,
                'password' => Hash::make($password),
                'email_verified_at' => now(),
                'activated_at' => now(),
                'superadmin' => 1,
            ])->save();
        });

        Nova::mainMenu(function (Request $request) {
            return [
                // MenuSection::make('Dashboard')->path('/dashboards/income')->icon('view-grid'),

                // MenuSection::make("Profiles",[
                //     MenuItem::resource(User::class),
                // ])->collapsable(),
                MenuSection::make("Menu",[
                    MenuItem::resource(Place::class),
                    MenuItem::resource(User::class),
                    MenuItem::resource(Advertisement::class)
                ])->collapsable()->icon('menu'),
                MenuSection::make("Report",[
                    MenuItem::resource(Assignment::class),
                    MenuItem::lens(Assignment::class, \App\Nova\Lenses\RequestedCars::class)->canSee(function ($request){
                        $user = auth()->guard('web')->user();
                        $is_manager = false;
                        foreach($user->places as $place){
                            if($place->pivot->permissions->contains("MANAGER")){
                                $is_manager = true;
                                break;
                            }
                        }
                        return
                            $user->superadmin
                            ||
                            $is_manager;
                    }),
                    MenuItem::lens(Assignment::class, \App\Nova\Lenses\GuestCars::class)->canSee(function ($request){
                        $user = auth()->guard('web')->user();
                        $is_manager = false;
                        foreach($user->places as $place){
                            if($place->pivot->permissions->contains("MANAGER")){
                                $is_manager = true;
                                break;
                            }
                        }
                        return
                            $user->superadmin
                            ||
                            $is_manager;
                    }),
                    MenuItem::make('Income')->path('/dashboards/income'),
                    MenuItem::make('Parking')->path('/dashboards/parking'),
                    MenuItem::lens(User::class, \App\Nova\Lenses\SoonExpiredUsers::class)->canSee(function ($request){
                            $user = auth()->guard('web')->user();
                            $is_manager = false;
                            foreach($user->places as $place){
                                if($place->pivot->permissions->contains("MANAGER")){
                                    $is_manager = true;
                                    break;
                                }
                            }
                            return
                                $user->superadmin
                                ||
                                $is_manager;
                        }),


                ])->collapsable(),
            ];
        });

        Nova::footer(function ($request) {
            return Blade::render('
                    <div class="mt-8 leading-normal text-xs text-gray-500 space-y-1">
                        <p class="text-center">
                            Powered By <a class="link-default">Tasree</a>
                        </p>
                    </div> 
            ');
        });
    }

    /**
     * Register the Nova routes.
     *
     * @return void
     */
    protected function routes()
    {
        Nova::routes()
                ->withAuthenticationRoutes()
                ->withPasswordResetRoutes()
                ->register();
    }

    /**
     * Register the Nova gate.
     *
     * This gate determines who can access Nova in non-local environments.
     *
     * @return void
     */
    protected function gate()
    {
        Gate::define('viewNova', function ($user) {
            return in_array($user->email, [
                //
            ]);
        });
    }

    /**
     * Get the dashboards that should be listed in the Nova sidebar.
     *
     * @return array
     */
    protected function dashboards()
    {
        return [
            new \App\Nova\Dashboards\Main,
            new \App\Nova\Dashboards\Income,
            new \App\Nova\Dashboards\Parking,
        ];
    }

    /**
     * Get the tools that should be listed in the Nova sidebar.
     *
     * @return array
     */
    public function tools()
    {
        return [
            new LaravelNovaCsvImport,
        ];
    }

    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        //
    }
}
